@@ -1,22 +1,24 @@
 package Workflow::Action;
 
-# $Id: Action.pm 302 2007-07-03 14:54:51Z jonasbn $
+# $Id: Action.pm 475 2009-07-19 15:31:08Z jonasbn $
 
 # Note: we may implement a separate event mechanism so that actions
 # can trigger other code (use 'Class::Observable'? read observations
 # from database?)
 
+use warnings;
 use strict;
 use base qw( Workflow::Base );
-use Log::Log4perl     qw( get_logger );
+use Log::Log4perl qw( get_logger );
 use Workflow::Action::InputField;
 use Workflow::Validator::HasRequiredField;
 use Workflow::Factory qw( FACTORY );
+use Carp qw(croak);
 
-$Workflow::Action::VERSION = '1.09';
+$Workflow::Action::VERSION = '1.10';
 
 my @FIELDS = qw( name class description );
-__PACKAGE__->mk_accessors( @FIELDS );
+__PACKAGE__->mk_accessors(@FIELDS);
 
 ####################
 # INPUT FIELDS
@@ -27,17 +29,17 @@ sub add_fields {
 }
 
 sub required_fields {
-    my ( $self ) = @_;
+    my ($self) = @_;
     return grep { $_->requirement() eq 'required' } @{ $self->{_fields} };
 }
 
 sub optional_fields {
-    my ( $self ) = @_;
+    my ($self) = @_;
     return grep { $_->requirement() eq 'optional' } @{ $self->{_fields} };
 }
 
 sub fields {
-    my ( $self ) = @_;
+    my ($self) = @_;
     return @{ $self->{_fields} };
 }
 
@@ -47,17 +49,21 @@ sub fields {
 sub add_validators {
     my ( $self, @validator_info ) = @_;
     my @validators = ();
-    foreach my $conf ( @validator_info ) {
+    foreach my $conf (@validator_info) {
         my $validator = FACTORY->get_validator( $conf->{name} );
-        my @args = $self->normalize_array( $conf->{arg} );
-        push @validators, { validator => $validator,
-                            args      => \@args };
+        my @args      = $self->normalize_array( $conf->{arg} );
+        push @validators,
+            {
+            validator => $validator,
+            args      => \@args
+            };
     }
     push @{ $self->{_validators} }, @validators;
 }
 
 sub get_validators {
-    my ( $self ) = @_;
+    my ($self) = @_;
+    return () if (not defined $self->{_validators});
     return @{ $self->{_validators} };
 }
 
@@ -67,27 +73,26 @@ sub validate {
     return unless ( scalar @validators );
 
     my $context = $wf->context;
-    foreach my $validator_info ( @validators ) {
-        my $validator = $validator_info->{validator};
-        my $args      = $validator_info->{args};
-        my @runtime_args = ( $wf );
-        foreach my $arg ( @{ $args } ) {
+    foreach my $validator_info (@validators) {
+        my $validator    = $validator_info->{validator};
+        my $args         = $validator_info->{args};
+        my @runtime_args = ($wf);
+        foreach my $arg ( @{$args} ) {
             if ( $arg =~ /^\$(.*)$/ ) {
-                push @runtime_args, $context->param( $1 );
-            }
-            else {
+                push @runtime_args, $context->param($1);
+            } else {
                 push @runtime_args, $arg;
             }
         }
-        $validator->validate( @runtime_args );
+        $validator->validate(@runtime_args);
     }
 }
 
 # Subclasses override...
 
-sub execute  {
+sub execute {
     my ( $self, $wf ) = @_;
-    die "Class ", ref( $self ), " must implement 'execute()'\n";
+    croak "Class ", ref($self), " must implement 'execute()'\n";
 }
 
 ########################################
@@ -97,7 +102,7 @@ sub init {
     my ( $self, $wf, $params ) = @_;
 
     # So we don't destroy the original...
-    my %copy_params = %{ $params };
+    my %copy_params = %{$params};
 
     $self->class( $copy_params{class} );
     $self->name( $copy_params{name} );
@@ -105,33 +110,33 @@ sub init {
 
     ## init normal fields
     my @fields = $self->normalize_array( $copy_params{field} );
-    foreach my $field_info ( @fields ) {
-        $self->add_fields( Workflow::Action::InputField->new( $field_info ) );
+    foreach my $field_info (@fields) {
+        $self->add_fields( Workflow::Action::InputField->new($field_info) );
     }
 
     ## establish validator for fields with is_required="yes"
     @fields = $self->required_fields();
-    my $validator = Workflow::Validator::HasRequiredField->new (
-                    {
-                        name  => 'HasRequiredField for is_required fields',
-                        class => 'Workflow::Validator::HasRequiredField'
-                    });
+    my $validator = Workflow::Validator::HasRequiredField->new(
+        {   name  => 'HasRequiredField for is_required fields',
+            class => 'Workflow::Validator::HasRequiredField'
+        }
+    );
     my @args = ();
-    foreach my $field ( @fields ) {
-        next if (not $field); ## empty @fields array
+    foreach my $field (@fields) {
+        next if ( not $field );    ## empty @fields array
         push @args, $field->name();
     }
     push @{ $self->{_validators} },
-         {
-             validator => $validator,
-             args      => \@args
-         };
+        {
+        validator => $validator,
+        args      => \@args
+        };
 
     ## init normal validators
     my @validator_info = $self->normalize_array( $copy_params{validator} );
-    $self->add_validators( @validator_info );
+    $self->add_validators(@validator_info);
 
-    delete @copy_params{ qw( class name description field validator ) };
+    delete @copy_params{qw( class name description field validator )};
 
     # everything else is just a passthru param
 
@@ -140,7 +145,6 @@ sub init {
     }
 }
 
-
 1;
 
 __END__
@@ -149,6 +153,10 @@ __END__
 
 Workflow::Action - Base class for Workflow actions
 
+=head1 VERSION
+
+This documentation describes version 1.09 of this package
+
 =head1 SYNOPSIS
 
  # Configure the Action...
@@ -165,36 +173,36 @@ Workflow::Action - Base class for Workflow actions
  </action>
 
  # Define the action
- 
+
  package MyApp::Action::CreateUser;
- 
+
  use base qw( Workflow::Action );
  use Workflow::Exception qw( workflow_error );
- 
+
  sub execute {
      my ( $self, $wf ) = @_;
      my $context = $wf->context;
- 
+
      # Since 'username' and 'email' have already been validated we
      # don't need to check them for uniqueness, well-formedness, etc.
- 
+
      my $user = eval {
          User->create({ username => $context->param( 'username' ),
                         email    => $context->param( 'email' ) })
      };
- 
+
      # Wrap all errors returned...
- 
+
      if ( $@ ) {
          workflow_error
-             "Cannot create new user with name '", $context->param( 'username' ), "': $@";
+             "Cannot create new user with name '", $context->param( 'username' ), "': $EVAL_ERROR";
      }
- 
+
      # Set the created user in the context for the application and/or
      # other actions (observers) to use
- 
+
      $context->param( user => $user );
- 
+
      # return the username since it might be used elsewhere...
      return $user->username;
  }
@@ -204,6 +212,35 @@ Workflow::Action - Base class for Workflow actions
 This is the base class for all Workflow Actions. You do not have to
 use it as such but it is strongly recommended.
 
+=head1 CONFIGURATION
+
+You configure your actions and map them to a specific module in your actions
+configuration file using the syntax
+above and that shown in L<Workflow>. In some cases, you'll have actions
+that apply to all workflows. In more elaborate configurations, you may have
+one workflow server loading multiple workflows and multiple actions for each.
+In these
+cases, you'll have multiple workflow types and you may want actions
+with the same names to have different behaviors for each type.
+
+For example, you may have a workflow type Ticket and another type Order_Parts.
+They both may have a Submit action, but you'll want the Submit to be different
+for each.
+
+You can specify a type in your actions configuration to associate that action
+with that workflow type. If you don't provide a type, the action is available
+to all types. For example:
+
+  <actions>
+    <type>Ticket</type>
+    <description>Actions for the Ticket workflow only.</description>
+    <action name="TIX_NEW"
+           class="TestApp::Action::TicketCreate">
+  ...Addtional configuration...
+
+The type must match an existing workflow type or the action will never
+be called.
+
 =head1 OBJECT METHODS
 
 =head2 Public Methods
