@@ -1,19 +1,23 @@
 package Workflow::Validator::MatchesDateFormat;
 
+# $Id: MatchesDateFormat.pm 454 2009-01-12 10:04:02Z jonasbn $
+
+use warnings;
 use strict;
 use base qw( Workflow::Validator );
 use DateTime::Format::Strptime;
 use Workflow::Exception qw( configuration_error validation_error );
+use English qw( -no_match_vars );
+use Carp qw(carp);
 
 $Workflow::Validator::MatchesDateFormat::VERSION = '1.06';
 
-__PACKAGE__->mk_accessors( 'formatter' );
+__PACKAGE__->mk_accessors('formatter');
 
 sub _init {
     my ( $self, $params ) = @_;
     unless ( $params->{date_format} ) {
-        configuration_error
-            "You must define a value for 'date_format' in ",
+        configuration_error "You must define a value for 'date_format' in ",
             "declaration of validator ", $self->name;
     }
     if ( ref $params->{date_format} ) {
@@ -22,25 +26,31 @@ sub _init {
             "declaration of validator ", $self->name;
     }
     my $formatter = DateTime::Format::Strptime->new(
-                             pattern => $params->{date_format},
-                             on_error => 'undef' );
-    $self->formatter( $formatter );
+        pattern  => $params->{date_format},
+        on_error => 'undef'
+    );
+    $self->formatter($formatter);
 }
 
 sub validate {
     my ( $self, $wf, $date_string ) = @_;
-    return unless ( $date_string );
+    return unless ($date_string);
 
     # already converted!
-    if ( ref ( $date_string ) and UNIVERSAL::isa( $date_string, 'DateTime' ) ) {
+    if ( ref $date_string and eval { $date_string->isa('DateTime'); } ) {
         return;
     }
 
-    my $fmt = $self->formatter;
-    my $date_object = $fmt->parse_datetime( $date_string );
-    unless ( $date_object ) {
+    if ($EVAL_ERROR) {
+        carp 'Unable to assert DateTime or similar object';
+    }
+
+    my $fmt         = $self->formatter;
+    my $date_object = $fmt->parse_datetime($date_string);
+    unless ($date_object) {
         validation_error
-            "Date '$date_string' does not match required pattern '", $fmt->pattern, "'";
+            "Date '$date_string' does not match required pattern '",
+            $fmt->pattern, "'";
     }
 }
 
@@ -52,6 +62,10 @@ __END__
 
 Workflow::Validator::MatchesDateFormat - Ensure a stringified date matches a given pattern
 
+=head1 VERSION
+
+This documentation describes version 1.06 of this package
+
 =head1 SYNOPSIS
 
  <action name="CreateNews">
